* File Name: subset.prg
* Author:   Michael A. Cohen
*           Triton Systems Corporation
*           9300 Lee Highway
*           Fairfax, Va 22031
*           703-934-3900
*
*--------------------------------------------------------------------------
* Description: A Clipper version of Skip Moon's Subntx C function
* that allows you to create a subset (almost) of an NTX file.
*--------------------------------------------------------------------------



*--------------------------------------------------------------------------
* Make "SUBTEST" if not found   Taken from Moon's PRG, with minor mods.
*--------------------------------------------------------------------------
CLEAR

maxrecs = 2000

PUBLIC flds[3]

flds[1]="rec"
flds[2]="fname"
flds[3]="lname"
if ! file("SUBTEST.DBF")
    ? "Stand by while creating SUBTEST.DBF......"
    if ! file("tempstr.dbf")
        create tempstr
    endif
    use tempstr
    append blank
    replace field_name with "rec",;
            field_type with "N";
            field_len  with 5,;
            field_dec  with 0

    append blank
    replace field_name with "lname",;
            field_type with "C";
            field_len  with 20,;
            field_dec  with 0

    append blank
    replace field_name with "fname",;
            field_type with "C";
            field_len  with 20,;
            field_dec  with 0

    close databases
    create SUBTEST from tempstr
    delete file tempstr.dbf

    zap

    * Put some undesireable records at the beginning
    do while recno() < maxrecs/2
        append blank
        replace rec with recno(), ;
                lname with "Lower records",;
                fname with "Undesireable"
    enddo


    * Put some matching records in the middle
    for i = 1 to 30
        append blank
        replace rec with recno(),;
                lname with "Moo"+chr(randint(97,123)),;
                fname with "Desireable"
    next

    * Put some undesireable upper records
    do while recno() < maxrecs
        append blank
        replace rec with recno(),;
                lname with "Upper records",;
                fname with "Undesireable"
    enddo

endif
*--------------------------------------------------------------------------
* Open the file to index
*--------------------------------------------------------------------------
use subtest   && modified from Skip Moon's "Subntx" C function.
if .not.file("Subtest.ntx")
 ? "Creating Full index, please wait . . ."
 index on lname to subtest
 set index to
endif
go top
? "Creating Subset index, please wait . . ."
retval=subntx2("subtest","subtemp2","Moo")
if retval
 use subtest
 set index to subtemp2
 j=indexkey(0)
*--------------------------------------------------------------------------
* Set a filter to get rid of that "bottom of file" record.  This is not slow
* because there is only one record to get rid of.
*--------------------------------------------------------------------------
 set filter to &j = "Moo"
 go top
 dbedit()  && display the file.
 clear
else
 ? "Error occured when indexing."
endif
Return


Function subntx2
*--------------------------------------------------------------------------
* Takes an existing index and creates a subset index from it.
*--------------------------------------------------------------------------
parameter index1,indext,subset
*                         
*                         -SUBSET STRING
*                   
*                   THE NAME OF THE SUBSET INDEX (created)
*             
*             THE NAME OF THE EXISTING INDEX
private key1,first,retval,prerec
*--------------------------------------------------------------------------
* Get the key field from the existing index
*--------------------------------------------------------------------------
set index to (index1)
key1=indexkey(0)
if empty(key1)
 @ 0,0 say 'Error'
 Return(.F.)
endif
set index to
*--------------------------------------------------------------------------
* Since Clipper appears to use the "Go" command to get to the next record
* when indexing, we have to save the record position so we can skip with
* an index.
*--------------------------------------------------------------------------
prerec=0       && used to keep track of the position in the file.
first=.T.      && seek the first record in the subset once.
index on subfun("&key1.") to (indext)
set index to
*--------------------------------------------------------------------------
* remove the call to the "Subfun" from the index file.
*--------------------------------------------------------------------------
retval=funrem(indext,key1)

Return(retval)

Function subfun
*--------------------------------------------------------------------------
* Subset index function
*--------------------------------------------------------------------------
parameter tkey1

if .not.eof()
*--------------------------------------------------------------------------
* First time through, seek the first record in the subset.
*--------------------------------------------------------------------------
 if first
   set index to (index1)  && re-open the index that Clipper closed.
   seek subset
   first=.F.
   prerec=recno()  && saves the position in the file so we can skip with an index.
 else
*--------------------------------------------------------------------------
* Get the next record in the existing index
*--------------------------------------------------------------------------
   go prerec
   skip
   prerec=recno()
 endif
*--------------------------------------------------------------------------
* Check to see if this record is in the subset, if not, close the index
* and go to the bottom record. (I can't figure out a way to get rid of this
* record from the index, so it remains)
*--------------------------------------------------------------------------
 if &tkey1 # subset
  set index to
  go bottom
 endif
endif
return (&tkey1)

FUNCTION funrem
*--------------------------------------------------------------------------
* Removes the "SUBFUN" function from the index
*--------------------------------------------------------------------------
PARAMETER ntxname,ntxkey
PRIVATE handle,newpos,tlen,newkey,ntxname,ntxkey,retval
retval=.T.
*--------------------------------------------------------------------------
* Open the index file with low level functions
*--------------------------------------------------------------------------
ntxname=TRIM(ntxname)+".NTX"
handle=Fopen(ntxname,2)
IF Ferror() # 0
    retval=.F.
ENDIF
if retval
 newpos=Fseek(handle,22,0)           && Starting position for the index key.
 tlen=LEN(ntxkey)+10                 && 10 is the length of => SUBFUN("")
 newkey=ntxkey+replicate(CHR(0),10)  && pad the key with Nulls
 IF newpos = 22
     Fwrite(handle,newkey,tlen)
 ENDIF
 Fclose(handle)
endif
RETURN(retval)

FUNCTION Randint
* Create a random integer between N and M-1
PARAM N,M
IF TYPE('xrndseed')=='U'
   PUBLIC xrndseed
   xrndseed = .123456789 + VAL(SUBSTR(TIME(),7,2))/100
ENDIF
xrndseed = (xrndseed * 31415821 + 1) / 1000000
xrndseed = xrndseed - INT(xrndseed)
RETURN INT(xrndseed * (M-N) + N)
